%%% Simple predicates on lists
%%% Author: Frank Pfenning
%%% 
%%% If a predicate p is already built-in, the name
%%% has been modified to p_ to allow this to compile

% list_(Xs) iff Xs is a list
% Same as built-in list(Xs)
list_([]).
list_([X|Xs]) :- list_(Xs).

% nat(N) iff N is a natural number in unary notation
nat(z).
nat(s(N)) :- nat(N).

% natlist(N) iff N is a list of natural numbers
natlist([]).
natlist([N|Ns]) :- nat(N), natlist(Ns).

% plist(P, Xs) iff Xs is a list where each element satisfies P
plist(P, []).
plist(P, [X|Xs]) :- G=..[P, X], call(G), plist(P, Xs).
% P(X) or call(P(X)) works in some Prolog implementations,
% but not in GNU Prolog.
% Decompose into G=..[P, X] which sets G = P(X) and then call G




% member_(X, Ys) iff X is a member of the list Ys.


% prefix_(Xs, Ys) iff list Xs is a prefix of list Ys


% suffix_(Xs, Ys) iff list Xs if a suffix of list Ys



% append_(Xs, Ys, Zs) iff Zs is the result of appending list Ys to list Xs



% alternative definitions of prefix and suffix using append_.


% sublist_(Xs, Ys) iff Xs is a sublist of Ys


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% length_(Xs, N) iff Xs is a list of length N.
